@@ -68,16 +68,18 @@ ll_sdmTMB <- function(object, withheld_y, withheld_mu) {
 
 #' Cross validation with sdmTMB models
 #'
-#' Facilitates cross validation with sdmTMB models. Returns the log likelihood
-#' of left-out data, which is similar in spirit to the ELPD (expected log
-#' pointwise predictive density). The function has an option for
-#' leave-future-out cross validation. By default, the function creates folds
-#' randomly but folds can be manually assigned via the `fold_ids` argument.
+#' Performs k-fold or leave-future-out cross validation with sdmTMB models.
+#' Returns the sum of log likelihoods of held-out data (log predictive density),
+#' which can be used to compare models—higher values indicate better
+#' out-of-sample prediction. By default, creates folds randomly and stratified
+#' by time (set a seed for reproducibility), but folds can be manually assigned
+#' via `fold_ids`. See Ward and Anderson (2025) in the References and the
+#' [cross-validation vignette](https://sdmTMB.github.io/sdmTMB/articles/cross-validation.html).
 #'
 #' @param formula Model formula.
 #' @param data A data frame.
-#' @param mesh Output from [make_mesh()]. If supplied, the mesh will be constant
-#'   across folds.
+#' @param mesh Output from [make_mesh()]. If supplied, the same mesh will be
+#'   used for all folds. This is faster and usually what you want.
 #' @param mesh_args Arguments for [make_mesh()]. If supplied, the mesh will be
 #'   reconstructed for each fold.
 #' @param time The name of the time column. Leave as `NULL` if this is only
@@ -86,44 +88,48 @@ ll_sdmTMB <- function(object, withheld_y, withheld_mu) {
 #' @param fold_ids Optional vector containing user fold IDs. Can also be a
 #'   single string, e.g. `"fold_id"` representing the name of the variable in
 #'   `data`. Ignored if `lfo` is TRUE
-#' @param lfo Whether to implement leave-future-out (LFO) cross validation where
-#'   data are used to predict future folds. `time` argument in [sdmTMB()] must
-#'   be specified. See Details section below.
-#' @param lfo_forecast If `lfo = TRUE`, number of time steps to forecast. Time
-#'   steps 1, ..., T are used to predict T + `lfo_forecast` and the last
-#'   forecasted time step is used for validation. See Details section below.
+#' @param lfo Logical. Use leave-future-out (LFO) cross validation? If `TRUE`,
+#'   data from earlier time steps are used to predict future time steps. The
+#'   `time` argument must be specified. See Details section below.
+#' @param lfo_forecast If `lfo = TRUE`, number of time steps ahead to forecast.
+#'   For example, `lfo_forecast = 1` means fitting to time steps 1 to T and
+#'   validating on T + 1. See Details section below.
 #' @param lfo_validations If `lfo = TRUE`, number of times to step through the
-#'   LFOCV process. Defaults to 5. See Details section below.
+#'   LFO process (i.e., number of validation folds). Defaults to 5. See Details
+#'   section below.
 #' @param parallel If `TRUE` and a [future::plan()] is supplied, will be run in
 #'   parallel.
 #' @param use_initial_fit Fit the first fold and use those parameter values
 #'   as starting values for subsequent folds? Can be faster with many folds.
+#' @param save_models Logical. If `TRUE` (default), the fitted model object for
+#'   each fold is stored in the output. If `FALSE`, models are not saved, which
+#'   can substantially reduce memory usage for large datasets or many folds.
+#'   When `FALSE`, functions that require access to the fitted models (e.g.,
+#'   [tidy()], [cv_to_waywiser()]) will not work.
 #' @param future_globals A character vector of global variables used within
 #'   arguments if an error is returned that \pkg{future.apply} can't find an
 #'   object. This vector is appended to `TRUE` and passed to the argument
 #'   `future.globals` in [future.apply::future_lapply()]. Useful if global
 #'   objects are used to specify arguments like priors, families, etc.
-#' @param ... All other arguments required to run [sdmTMB()] model with the
-#'   exception of `weights`, which are used to define the folds.
+#' @param ... All other arguments required to run the [sdmTMB()] model. The
+#'   `weights` argument is supported and will be combined with the internal
+#'   fold-assignment mechanism (held-out data are assigned weight 0).
 #'
 #' @export
 #' @return
 #' A list:
-#' * `data`: Original data plus columns for fold ID, CV predicted value,
-#'           and CV log likelihood.
-#' * `models`: A list of models; one per fold.
-#' * `fold_loglik`: Sum of left-out log likelihoods per fold. More positive
-#'   values are better.
-#' * `sum_loglik`: Sum of `fold_loglik` across all left-out data. More positive
-#'   values are better.
-#' * `pdHess`: Logical vector: Hessian was invertible each fold?
-#' * `converged`: Logical: all `pdHess` `TRUE`?
-#' * `max_gradients`: Max gradient per fold.
-#'
-#' Prior to \pkg{sdmTMB} version '0.3.0.9002', `elpd` was incorrectly returned as
-#' the log average likelihood, which is another metric you could compare models
-#' with, but not ELPD. For maximum likelihood, [ELPD is equivalent in spirit to
-#' the sum of the log likelihoods](https://github.com/pbs-assess/sdmTMB/issues/235).
+#' * `data`: Original data plus columns for fold ID (`cv_fold`), CV predicted
+#'   value (`cv_predicted`), CV log likelihood (`cv_loglik`), and CV deviance
+#'   residuals (`cv_deviance_resid`).
+#' * `models`: A list of fitted models, one per fold. `NULL` if `save_models = FALSE`.
+#' * `fold_loglik`: Sum of log likelihoods of held-out data per fold (log
+#'   predictive density per fold). More positive values indicate better
+#'   out-of-sample prediction.
+#' * `sum_loglik`: Sum of `fold_loglik` across all folds (total log predictive
+#'   density). Use this to compare models—more positive values are better.
+#' * `pdHess`: Logical vector: was the Hessian positive definite for each fold?
+#' * `converged`: Logical: did all folds converge (all `pdHess` `TRUE`)?
+#' * `max_gradients`: Maximum absolute gradient for each fold.
 #'
 #' @details
 #' **Parallel processing**
@@ -160,6 +166,12 @@ ll_sdmTMB <- function(object, withheld_y, withheld_mu) {
 #'
 #' See example below.
 #'
+#' @references
+#'
+#' Ward, E.J., and S.C. Anderson. 2025. Approximating spatial processes with
+#' too many knots degrades the quality of probabilistic predictions.
+#' bioRxiv 2025.11.14.688354. \doi{10.1101/2025.11.14.688354}.
+#'
 #' @examples
 #' mesh <- make_mesh(pcod, c("X", "Y"), cutoff = 25)
 #'
@@ -222,6 +234,7 @@ sdmTMB_cv <- function(
     lfo_validations = 5,
     parallel = TRUE,
     use_initial_fit = FALSE,
+    save_models = TRUE,
     future_globals = NULL,
     ...) {
   if (k_folds < 1) cli_abort("`k_folds` must be >= 1.")
@@ -251,7 +264,7 @@ sdmTMB_cv <- function(
       }
     } else {
       dd <- lapply(split(data, data[[time]]), function(x) {
-        x$cv_fold <- sample(rep(seq(1L, k_folds), nrow(x)), size = nrow(x))
+        x$cv_fold <- sample(rep(seq(1L, k_folds), length.out = nrow(x)), size = nrow(x))
         x
       })
       data <- do.call(rbind, dd)
@@ -283,9 +296,20 @@ sdmTMB_cv <- function(
   }
 
   dot_args <- as.list(substitute(list(...)))[-1L]
+
+  # Extract user-supplied weights if provided
   if ("weights" %in% names(dot_args)) {
-    cli_abort("`weights` cannot be specified within sdmTMB_cv().")
+    user_weights <- eval(dot_args$weights, envir = parent.frame())
+    if (length(user_weights) != nrow(data)) {
+      cli_abort("`weights` must have the same length as the number of rows in `data`.")
+    }
+    if (any(user_weights <= 0)) {
+      cli_abort("`weights` must be positive (> 0).")
+    }
+  } else {
+    user_weights <- rep(1, nrow(data))
   }
+
   if ("offset" %in% names(dot_args)) {
     if (!is.character(dot_args$offset)) {
       cli_abort("Please use a character value for 'offset' (indicating the column name) for cross validation.")
@@ -297,11 +321,14 @@ sdmTMB_cv <- function(
 
   if (k_folds > 1) {
     # data in kth fold get weight of 0:
-    weights <- ifelse(data$cv_fold == 1L, 0, 1)
+    fold_weights <- ifelse(data$cv_fold == 1L, 0, 1)
   } else {
-    weights <- rep(1, nrow(data))
+    fold_weights <- rep(1, nrow(data))
   }
-  if (lfo) weights <- ifelse(data$cv_fold == 1L, 1, 0)
+  if (lfo) fold_weights <- ifelse(data$cv_fold == 1L, 1, 0)
+
+  # Combine user weights with fold weights
+  weights <- user_weights * fold_weights
 
   if (use_initial_fit) {
     # run model on first fold to get starting values:
@@ -312,17 +339,20 @@ sdmTMB_cv <- function(
       } else {
         dat_fit <- data[data$cv_fold != 1L, , drop = FALSE]
       }
-
+      # Create mesh on training data, then update with full data
       mesh_args[["data"]] <- dat_fit
+      mesh_train <- do.call(make_mesh, mesh_args)
+      mesh_args[["data"]] <- data
+      mesh_args[["mesh"]] <- mesh_train$mesh
       mesh <- do.call(make_mesh, mesh_args)
     } else {
       mesh <- spde
-      dat_fit <- data
     }
     dot_args <- list(dot_args)[[1]]
     dot_args$offset <- NULL
+    dot_args$weights <- NULL
     .args <- c(list(
-      data = dat_fit, formula = formula, time = time, mesh = mesh,
+      data = data, formula = formula, time = time, mesh = mesh,
       weights = weights, offset = .offset
     ), dot_args)
     fit1 <- do.call(sdmTMB, .args)
@@ -330,11 +360,13 @@ sdmTMB_cv <- function(
 
   fit_func <- function(k) {
     if (lfo) {
-      weights <- ifelse(data$cv_fold <= k, 1, 0)
+      fold_weights <- ifelse(data$cv_fold <= k, 1, 0)
     } else {
       # data in kth fold get weight of 0:
-      weights <- ifelse(data$cv_fold == k, 0, 1)
+      fold_weights <- ifelse(data$cv_fold == k, 0, 1)
     }
+    # Combine user weights with fold weights
+    weights <- user_weights * fold_weights
 
     if (k == 1L && use_initial_fit) {
       object <- fit1
@@ -345,17 +377,21 @@ sdmTMB_cv <- function(
         } else {
           dat_fit <- data[data$cv_fold != k, , drop = FALSE]
         }
+        # Create mesh on training data, then update with full data
         mesh_args[["data"]] <- dat_fit
+        mesh_train <- do.call(make_mesh, mesh_args)
+        mesh_args[["data"]] <- data
+        mesh_args[["mesh"]] <- mesh_train$mesh
         mesh <- do.call(make_mesh, mesh_args)
       } else {
         mesh <- spde
-        dat_fit <- data
       }
       dot_args <- as.list(substitute(list(...)))[-1L] # re-evaluate here! issue #54
       dot_args <- list(...)
       dot_args$offset <- NULL
+      dot_args$weights <- NULL
       args <- c(list(
-        data = dat_fit, formula = formula, time = time, mesh = mesh, offset = .offset,
+        data = data, formula = formula, time = time, mesh = mesh, offset = .offset,
         weights = weights, previous_fit = if (use_initial_fit) fit1 else NULL
       ), dot_args)
       object <- do.call(sdmTMB, args)
@@ -369,6 +405,7 @@ sdmTMB_cv <- function(
 
     # FIXME: only use TMB report() below to be faster!
     # predict for withheld data:
+    # cli_inform("Testing on data fold {k}.")
     predicted <- predict(object, newdata = cv_data, type = "response",
       offset = if (!is.null(.offset)) cv_data[[.offset]] else rep(0, nrow(cv_data)))
 
@@ -377,13 +414,36 @@ sdmTMB_cv <- function(
     withheld_y <- predicted[[response]]
     withheld_mu <- cv_data$cv_predicted
 
+    # Calculate deviance residuals for held-out data
+    dev_resids <- tryCatch({
+      residuals(object, type = "deviance")
+    }, error = function(e) NULL)
+
+    # Match deviance residuals to held-out data
+    if (!is.null(dev_resids) && !all(dev_resids == 0)) {
+      # Get the row indices from cv_data (held-out data)
+      cv_order <- cv_data[["_sdm_order_"]]
+      # Get the row indices from object$data (fitted data)
+      obj_order <- object$data[["_sdm_order_"]]
+      # Match held-out indices to fitted data indices
+      match_idx <- match(cv_order, obj_order)
+      # Extract corresponding deviance residuals
+      cv_data$cv_deviance_resid <- dev_resids[match_idx]
+    } else {
+      # Deviance residuals not available for this family
+      cv_data$cv_deviance_resid <- NA_real_
+    }
+
     # FIXME: get LFO working with the TMB report() option below!
     # calculate log likelihood for each withheld observation:
     # trickery to get the log likelihood of the withheld data directly
     # from the TMB report():
     if (!lfo) {
       tmb_data <- object$tmb_data
-      tmb_data$weights_i <- ifelse(tmb_data$weights_i == 1, 0, 1) # reversed
+
+      # Reverse weights: training (weight != 0) → 0, held-out (weight == 0) → user_weight
+      tmb_data$weights_i <- ifelse(tmb_data$weights_i == 0, user_weights, 0)
+
       new_tmb_obj <- TMB::MakeADFun(
         data = tmb_data,
         parameters = get_pars(object),
@@ -395,7 +455,8 @@ sdmTMB_cv <- function(
       lp <- object$tmb_obj$env$last.par.best
       r <- new_tmb_obj$report(lp)
       cv_loglik <- -1 * r$jnll_obs
-      cv_data$cv_loglik <- cv_loglik[tmb_data$weights_i == 1]
+      # Extract log-likelihoods for held-out observations (where reversed weights > 0)
+      cv_data$cv_loglik <- cv_loglik[tmb_data$weights_i > 0]
     } else { # old method; doesn't work with delta models!
       cv_data$cv_loglik <- ll_sdmTMB(object, withheld_y, withheld_mu)
     }
@@ -407,7 +468,7 @@ sdmTMB_cv <- function(
 
     list(
       data = cv_data,
-      model = object,
+      model = if (save_models) object else NULL,
       pdHess = object$sd_report$pdHess,
       max_gradient = max(abs(object$gradients)),
       bad_eig = object$bad_eig
@@ -442,7 +503,7 @@ sdmTMB_cv <- function(
     }
   }
 
-  models <- lapply(out, `[[`, "model")
+  models <- if (save_models) lapply(out, `[[`, "model") else NULL
   data <- lapply(out, `[[`, "data")
   fold_cv_ll <- vapply(data, function(.x) sum(.x$cv_loglik), FUN.VALUE = numeric(1L))
   data <- do.call(rbind, data)
@@ -473,18 +534,24 @@ log_sum_exp <- function(x) {
 #' @export
 #' @import methods
 print.sdmTMB_cv <- function(x, ...) {
-  nmods <- length(x$models)
   nconverged <- sum(x$pdHess)
-  cat(paste0("Cross validation of sdmTMB models with ", nmods, " folds.\n"))
+  nfolds <- length(x$pdHess)
+  cat(paste0("Cross validation of sdmTMB models with ", nfolds, " folds.\n"))
   cat("\n")
-  cat("Summary of the first fold model fit:\n")
-  cat("\n")
-  print(x$models[[1]])
-  cat("\n")
-  cat("Access the rest of the models in a list element named `models`.\n")
-  cat("E.g. `object$models[[2]]` for the 2nd fold model fit.\n")
-  cat("\n")
-  cat(paste0(nconverged, " out of ", nmods, " models are consistent with convergence.\n"))
+  if (is.null(x$models)) {
+    cat("Models were not saved (save_models = FALSE).\n")
+    cat("Only prediction and log likelihood results are available.\n")
+    cat("\n")
+  } else {
+    cat("Summary of the first fold model fit:\n")
+    cat("\n")
+    print(x$models[[1]])
+    cat("\n")
+    cat("Access the rest of the models in a list element named `models`.\n")
+    cat("E.g. `object$models[[2]]` for the 2nd fold model fit.\n")
+    cat("\n")
+  }
+  cat(paste0(nconverged, " out of ", nfolds, " models are consistent with convergence.\n"))
   cat("Figure out which folds these are in the `converged` list element.\n")
   cat("\n")
   cat(paste0("Out-of-sample log likelihood for each fold: ", paste(round(x$fold_loglik, 2), collapse = ", "), ".\n"))
@@ -494,3 +561,135 @@ print.sdmTMB_cv <- function(x, ...) {
   cat("More positive values imply better out-of-sample prediction.\n")
   cat("Access this value in the `sum_loglik` list element.\n")
 }
+
+#' Convert [sdmTMB_cv()] objects to \pkg{sf} format for spatial assessment with \pkg{waywiser}
+#'
+#' @description
+#' `r lifecycle::badge("experimental")`
+#' Converts cross-validation results to an [sf::sf()] object for use with
+#' spatial model assessment tools such as those in the \pkg{waywiser} package.
+#' This enables multi-scale spatial assessment of model predictions.
+#'
+#' @param object An object of class `sdmTMB_cv` from [sdmTMB_cv()].
+#' @param ll_names Column names for longitude and latitude in the original data.
+#'   **Note the order: longitude first, then latitude.**
+#' @param ll_crs The coordinate reference system (CRS) for the `ll_names` columns.
+#'   Defaults to `4326` (WGS84 lon/lat).
+#' @param utm_crs The projected coordinate reference system (CRS) for the output
+#'   sf object. By default (if you're feeling lucky!) automatically detected
+#'   using [get_crs()] based on `ll_names`. Can be manually specified as an
+#'   EPSG code (e.g., `32609`) or any format accepted by [sf::st_crs()].
+#'
+#' @return An [sf::sf()] object with POINT geometry containing:
+#' \describe{
+#'   \item{truth}{The observed response values}
+#'   \item{estimate}{The cross-validated predictions}
+#'   \item{geometry}{Spatial point locations}
+#' }
+#'
+#' @details
+#' This function is particularly useful for assessing spatial models at multiple
+#' scales using the \pkg{waywiser} package. After converting to sf format, you
+#' can use functions like [waywiser::ww_multi_scale()] to evaluate how model
+#' performance changes when predictions are aggregated to different spatial
+#' scales.
+#'
+#' For delta/hurdle models, the combined predictions are returned (e.g., the
+#' product of the encounter probability and positive catch rate).
+#'
+#' @seealso [sdmTMB_cv()], [get_crs()], \url{https://sdmTMB.github.io/sdmTMB/articles/cross-validation.html}
+#'
+#' @export
+#' @examplesIf require("sf", quietly = TRUE) && require("waywiser", quietly = TRUE)
+#' mesh <- make_mesh(pcod_2011, c("X", "Y"), cutoff = 12)
+#'
+#' # Run cross-validation
+#' set.seed(123)
+#' m_cv <- sdmTMB_cv(
+#'   density ~ depth_scaled,
+#'   data = pcod_2011,
+#'   mesh = mesh,
+#'   family = tweedie(),
+#'   k_folds = 2
+#' )
+#'
+#' # Convert with default auto-detected CRS based on lon/lat columns:
+#' cv_sf <- cv_to_waywiser(m_cv, ll_names = c("lon", "lat"))
+#'
+#' # Or manually specify the desired UTM CRS:
+#' cv_sf <- cv_to_waywiser(m_cv, ll_names = c("lon", "lat"), utm_crs = 32609)
+#'
+#' # Use with waywiser for multi-scale assessment
+#' waywiser::ww_multi_scale(
+#'   cv_sf,
+#'   truth,    # column name (unquoted)
+#'   estimate, # column name (unquoted)
+#'   n = list(c(5, 5), c(2, 2)) # 5x5 and 2x2 grid blocks
+#' )
+cv_to_waywiser <- function(object,
+                           ll_names = c("longitude", "latitude"),
+                           ll_crs = 4326,
+                           utm_crs = get_crs(object$data, ll_names)) {
+  if (!requireNamespace("sf", quietly = TRUE)) {
+    cli_abort("The sf package must be installed to use this function.")
+  }
+
+  if (is.null(object$models)) {
+    cli_abort(c(
+      "Models were not saved during cross-validation.",
+      "i" = "Set `save_models = TRUE` in `sdmTMB_cv()` to use `cv_to_waywiser()`."
+    ))
+  }
+
+  # Get response variable name from the first model's formula
+  formulas <- object$models[[1]]$formula
+  if (is.list(formulas)) {
+    # Delta model - use first formula
+    formula <- formulas[[1]]
+  } else {
+    # Non-delta model
+    formula <- formulas
+  }
+  response <- get_response(formula)
+
+  # Extract data
+  cv_data <- object$data
+
+  # Check that required columns exist
+  if (!response %in% names(cv_data)) {
+    cli_abort(c(
+      "Response variable '{response}' not found in CV data.",
+      "i" = "This may indicate an issue with the sdmTMB_cv object."
+    ))
+  }
+  if (!"cv_predicted" %in% names(cv_data)) {
+    cli_abort(c(
+      "'cv_predicted' column not found in CV data.",
+      "i" = "This may indicate an issue with the sdmTMB_cv object."
+    ))
+  }
+  if (!all(ll_names %in% names(cv_data))) {
+    cli_abort(c(
+      "Longitude/latitude columns '{paste(ll_names, collapse = ', ')}' not found in CV data.",
+      "i" = "Check that `ll_names` matches your data column names."
+    ))
+  }
+
+  # Create data frame with required columns including lon/lat coordinates
+  point_df <- data.frame(
+    truth = cv_data[[response]],
+    estimate = cv_data$cv_predicted
+  )
+  # Add the lon/lat coordinate columns
+  point_df[[ll_names[1]]] <- cv_data[[ll_names[1]]]
+  point_df[[ll_names[2]]] <- cv_data[[ll_names[2]]]
+
+  # Convert to sf object with lon/lat CRS, then transform to UTM
+  points <- sf::st_as_sf(point_df,
+    coords = ll_names,
+    crs = ll_crs
+  )
+  points <- sf::st_transform(points, utm_crs)
+
+  points
+}
