@@ -1,36 +1,45 @@
-#' Extract a relative biomass/abundance index, center of gravity, or effective
-#' area occupied
+#' Extract a relative biomass/abundance index, center of gravity, effective
+#' area occupied, or weighted average
 #'
-#' @param obj Output from [predict.sdmTMB()] with `return_tmb_object = TRUE`.
-#'   Alternatively, if [sdmTMB()] was called with `do_index = TRUE` or if using
-#'   the helper function [get_index_split()], an object from [sdmTMB()].
-#' @param bias_correct Should bias correction be implemented [TMB::sdreport()]?
-#'   This is recommended to be `TRUE` for any final analyses, but one may wish
-#'   to set this to `FALSE` for slightly faster calculations while experimenting
-#'   with models.
+#' @param obj Output from [predict.sdmTMB()] with `return_tmb_object = TRUE`
+#'   (the usual case). Alternatively, if [sdmTMB()] was called with `do_index =
+#'   TRUE`, or if using [get_index_split()], an object from [sdmTMB()].
+#' @param bias_correct Should bias correction be implemented via
+#'   [TMB::sdreport()]? Bias correction accounts for the non-linear
+#'   transformation of random effects when calculating the index. Recommended to
+#'   be `TRUE` for final analyses, but can be set to `FALSE` for faster
+#'   calculation while experimenting with models. See Thorson and Kristensen
+#'   (2016) in the References.
 #' @param level The confidence level.
-#' @param area Grid cell area. A vector of length `newdata` from
-#'   [predict.sdmTMB()] *or* a value of length 1 which will be repeated
-#'   internally to match *or* a character value representing the column
-#'   used for area weighting.
+#' @param area Grid cell area for area weighting the index. Can be: (1) a
+#'   numeric vector of length `nrow(newdata)` with area for each grid cell, (2)
+#'   a single numeric value to apply to all grid cells, or (3) a character value
+#'   giving the column name in `newdata` containing areas.
 #' @param silent Silent?
 #' @param ... Passed to [TMB::sdreport()].
 #'
 #' @seealso [get_index_sims()]
 #' @return
 #' For `get_index()`:
-#' A data frame with a columns for time, estimate, lower and upper
-#' confidence intervals, log estimate, and standard error of the log estimate.
+#' A data frame with columns for time, estimate (area-weighted total abundance
+#' or biomass), lower and upper confidence intervals, log estimate, and standard
+#' error of the log estimate.
 #'
 #' For `get_cog()`:
-#' A data frame with a columns for time, estimate (center of gravity in x and y
-#' coordinates), lower and upper confidence intervals, and standard error of
-#' center of gravity coordinates.
+#' A data frame with columns for time, estimate (center of gravity: the
+#' abundance-weighted mean x and y coordinates), lower and upper confidence
+#' intervals, and standard error of center of gravity coordinates.
 #'
 #' For `get_eao()`:
-#' A data frame with a columns for time, estimate (effective area occupied; EAO),
-#' lower and upper confidence intervals,
-#' log EAO, and standard error of the log EAO estimates.
+#' A data frame with columns for time, estimate (effective area occupied: the
+#' area required if the population was spread evenly at the arithmetic mean
+#' density), lower and upper confidence intervals, log EAO, and standard error
+#' of the log EAO estimates.
+#'
+#' For `get_weighted_average()`:
+#' A data frame with columns for time, estimate (weighted average of the
+#' provided vector, weighted by predicted density), lower and upper confidence
+#' intervals, and standard error of the estimates.
 #'
 #' @references
 #'
@@ -116,6 +125,12 @@
 #' ggplot(eao, aes(year, est)) + geom_line() +
 #'   geom_ribbon(aes(ymin = lwr, ymax = upr), alpha = 0.4) +
 #'   ylim(0, NA)
+#'
+#' # weighted average (e.g., depth-weighted by biomass):
+#' wa <- get_weighted_average(predictions, vector = nd$depth)
+#' wa
+#' ggplot(wa, aes(year, est)) + geom_line() +
+#'   geom_ribbon(aes(ymin = lwr, ymax = upr), alpha = 0.4)
 #' }
 #' @export
 get_index <- function(obj, bias_correct = TRUE, level = 0.95, area = 1, silent = TRUE, ...)  {
@@ -233,6 +248,29 @@ get_cog <- function(obj, bias_correct = FALSE, level = 0.95, format = c("long",
   d
 }
 
+#' @rdname get_index
+#' @param vector A numeric vector of the same length as the prediction data,
+#'   containing the values to be averaged (e.g., depth, temperature).
+#' @export
+get_weighted_average <- function(obj, vector, bias_correct = FALSE, level = 0.95,
+  area = 1, silent = TRUE, ...)  {
+  # if offset is a character vector, use the value in the dataframe
+  if (is.character(area)) {
+    area <- obj$data[[area]]
+  }
+
+  if (is.null(vector)) {
+    cli_abort("A vector must be provided for weighted average calculation.")
+  }
+
+  d <- get_generic(obj, value_name = "weighted_avg",
+    bias_correct = bias_correct, level = level, trans = I, area = area,
+    vector = vector, ...)
+  d <- d[, names(d) != "trans_est", drop = FALSE]
+  d$type <- "weighted_average"
+  d
+}
+
 #' @rdname get_index
 #' @export
 get_eao <- function(obj,
@@ -255,7 +293,7 @@ get_eao <- function(obj,
 }
 
 get_generic <- function(obj, value_name, bias_correct = FALSE, level = 0.95,
-  trans = I, area = 1, silent = TRUE, ...) {
+  trans = I, area = 1, vector = NULL, silent = TRUE, ...) {
 
   # if offset is a character vector, use the value in the dataframe
   if (is.character(area)) {
@@ -264,12 +302,19 @@ get_generic <- function(obj, value_name, bias_correct = FALSE, level = 0.95,
 
   reinitialize(obj$fit_obj)
 
-  if ((!isTRUE(obj$do_index) && value_name[1] == "link_total") || value_name[1] == "cog_x" || value_name[[1]] == "log_eao") {
+  if ((!isTRUE(obj$do_index) && value_name[1] == "link_total") || value_name[1] == "cog_x" || value_name[[1]] == "log_eao" || value_name[1] == "weighted_avg") {
     if (is.null(obj[["obj"]])) {
       cli_abort(paste0("`obj` needs to be created with ",
         "`predict(..., return_tmb_object = TRUE).`"))
     }
 
+    nr1 <- nrow(obj$data)
+    nr2 <- nrow(obj$pred_tmb_data$proj_X_ij[[1]])
+    if (nr1 != nr2) {
+      cli_abort(c("Predicted data appears to be modified after prediction",
+        "i" ="Please filter `newdata` before predicting."))
+    }
+
     if (!"report" %in% names(obj$obj)) {
       cli_abort(c("It looks like the predict function was run without `newdata` specified.",
         "Re-run the predict function with `newdata` specified.")) #276
@@ -304,6 +349,16 @@ get_generic <- function(obj, value_name, bias_correct = FALSE, level = 0.95,
       tmb_data$calc_cog <- 1L
     if (value_name[1] == "log_eao")
       tmb_data$calc_eao <- 1L
+    if (value_name[1] == "weighted_avg") {
+      if (is.null(vector)) {
+        cli_abort("A vector must be provided for weighted average calculation.")
+      }
+      if (length(vector) != nrow(obj$pred_tmb_data$proj_X_ij[[1]])) {
+        cli_abort("`vector` should be of the same length as `nrow(newdata)`.")
+      }
+      tmb_data$proj_vector <- vector
+      tmb_data$calc_weighted_avg <- 1L
+    }
 
     pars <- get_pars(obj$fit_obj)
 
@@ -334,10 +389,11 @@ get_generic <- function(obj, value_name, bias_correct = FALSE, level = 0.95,
   }
   sr_est <- as.list(sr, "Estimate", report = TRUE)
 
-  if (bias_correct && value_name[[1]] %in% c("link_total", "cog_x")) {
+  if (bias_correct && value_name[[1]] %in% c("link_total", "cog_x", "weighted_avg")) {
     # extract and modify parameters
     if (value_name[[1]] == "link_total") .n <- length(sr_est$total)
     if (value_name[[1]] == "cog_x") .n <- length(sr_est$cog_x) * 2 # 2 b/c x and y
+    if (value_name[[1]] == "weighted_avg") .n <- length(sr_est$weighted_avg)
     pars[[eps_name]] <- rep(0, .n)
     new_values <- rep(0, .n)
     names(new_values) <- rep(eps_name, length(new_values))
@@ -356,7 +412,7 @@ get_generic <- function(obj, value_name, bias_correct = FALSE, level = 0.95,
     gradient <- new_obj2$gr(fixed)
     corrected_vals <- gradient[names(fixed) == eps_name]
   } else {
-    if (value_name[[1]] == "link_total" || value_name[[1]] == "cog_x")
+    if (value_name[[1]] == "link_total" || value_name[[1]] == "cog_x" || value_name[[1]] == "weighted_avg")
       cli_inform(c("Bias correction is turned off.", "
         It is recommended to turn this on for final inference."))
   }
@@ -367,7 +423,7 @@ get_generic <- function(obj, value_name, bias_correct = FALSE, level = 0.95,
   time_name <- obj$fit_obj$time
   names(d) <- c("trans_est", "se")
   if (bias_correct) {
-    if (value_name[[1]] == "cog_x") {
+    if (value_name[[1]] == "cog_x" || value_name[[1]] == "weighted_avg") {
       d$trans_est <- corrected_vals
     } else {
       d$trans_est <- log(corrected_vals)
@@ -379,6 +435,12 @@ get_generic <- function(obj, value_name, bias_correct = FALSE, level = 0.95,
   d$lwr <- as.numeric(trans(d$trans_est + stats::qnorm((1-level)/2) * d$se))
   d$upr <- as.numeric(trans(d$trans_est + stats::qnorm(1-(1-level)/2) * d$se))
 
+  # also grab natural space SE:
+  if ("link_total" %in% value_name) {
+    .total <- ssr[row.names(ssr) %in% "total", , drop = FALSE]
+    d$se_natural <- as.numeric(.total[,1])
+  }
+
   if ("pred_tmb_data" %in% names(obj)) { # standard case
     ii <- sort(unique(obj$pred_tmb_data$proj_year))
   } else { # fit with do_index = TRUE
@@ -406,5 +468,9 @@ get_generic <- function(obj, value_name, bias_correct = FALSE, level = 0.95,
   if ("do_index_time_missing_from_nd" %in% names(obj$fit_obj)) {
     d <- d[!d[[obj$fit_obj$time]] %in% obj$fit_obj$do_index_time_missing_from_nd, ,drop = FALSE]
   }
-  d[,c(time_name, 'est', 'lwr', 'upr', 'trans_est', 'se'), drop = FALSE]
+  if (!"link_total" %in% value_name) {
+    d[,c(time_name, 'est', 'lwr', 'upr', 'trans_est', 'se'), drop = FALSE]
+  } else {
+    d[,c(time_name, 'est', 'lwr', 'upr', 'trans_est', 'se', 'se_natural'), drop = FALSE]
+  }
 }
