@@ -155,3 +155,83 @@
 ###   expect_equal(gm$predicted, ge$predicted, tolerance = 1e-3)
 ### })
 ###
+
+# Test for issue #450: ggpredict() fails with multiple smoothers + an offset
+test_that("ggpredict works with multiple smoothers and offset (issue #450)", {
+  skip_on_cran()
+  skip_if_not_installed("ggeffects")
+
+  # Test 1: Basic model without smoothers
+  fit1 <- sdmTMB(
+    present ~ depth_scaled,
+    data = pcod_2011,
+    spatial = "off",
+    family = binomial()
+  )
+  g1 <- ggeffects::ggpredict(fit1, "depth_scaled[all]")
+  expect_s3_class(g1, "data.frame")
+  expect_true(nrow(g1) > 0)
+
+  # Test 2: Model with single smoother
+  fit2 <- sdmTMB(
+    present ~ s(depth_scaled, k = 5),
+    data = pcod_2011,
+    spatial = "off",
+    family = binomial()
+  )
+  g2 <- ggeffects::ggpredict(fit2, "depth_scaled[all]")
+  expect_s3_class(g2, "data.frame")
+  expect_true(nrow(g2) > 0)
+
+  # Test 3: Model with multiple smoothers
+  fit3 <- sdmTMB(
+    present ~ s(year, k = 3) + s(depth_scaled, k = 5),
+    data = pcod_2011,
+    spatial = "off",
+    family = binomial()
+  )
+  g3 <- ggeffects::ggpredict(fit3, "depth_scaled[all]")
+  expect_s3_class(g3, "data.frame")
+  expect_true(nrow(g3) > 0)
+
+  # Test 4: Model with offset and single smoother
+  fit4 <- sdmTMB(
+    catch_weight ~ s(depth, k = 3),
+    data = dogfish,
+    spatial = "off",
+    family = tweedie(),
+    offset = dogfish$area_swept
+  )
+  g4 <- suppressMessages(ggeffects::ggpredict(fit4, "depth[all]"))
+  expect_s3_class(g4, "data.frame")
+  expect_true(nrow(g4) > 0)
+
+  # Test 5: Model with offset and multiple smoothers (the main bug from #450)
+  fit5 <- sdmTMB(
+    catch_weight ~ s(depth, k = 3) + s(year, k = 3),
+    data = dogfish,
+    spatial = "off",
+    family = tweedie(),
+    offset = dogfish$area_swept
+  )
+  g5 <- suppressMessages(ggeffects::ggpredict(fit5, "depth[all]"))
+  expect_s3_class(g5, "data.frame")
+  expect_true(nrow(g5) > 0)
+  # Check that year is in the "Adjusted for" section
+  expect_true("year" %in% names(attributes(g5)$constant.values))
+
+  # Test 6: Model with tensor smoother
+  fit6 <- sdmTMB(
+    catch_weight ~ t2(depth, year),
+    data = dogfish,
+    spatial = "off",
+    family = tweedie()
+  )
+  g6 <- ggeffects::ggpredict(fit6, "depth[all]")
+  expect_s3_class(g6, "data.frame")
+  expect_true(nrow(g6) > 0)
+
+  # Test 7: Verify terms() method works correctly with smoothers
+  expect_equal(attr(terms(fit5), "term.labels"), c("depth", "year"))
+  expect_equal(attr(terms(fit6), "term.labels"), c("depth", "year"))
+})
